/*
Copyright (C) 2009 -  Claudio Salvadori
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/**
 *\file CXml.cpp
 *\brief  Xml parser
 */
 
#include "CXml.h"

#include <vector>
#include <string>
#include <stdio.h>
#include <sstream>

#include "DAQ_struct.h"

CXml::CXml(char* path_xml, struct SDef* def, vector<struct SMData>* metadata, string* title): x_def(def), x_meta(metadata), x_title(title), x_path(path_xml)
{
	//Init CXMLParser class object
	x_parser = new CXMLParser;
}

CXml::~CXml(){
	delete def_node;
	delete root_node;		
	delete stream_node;		
	delete pkt_node;		
	delete data_node;		
	delete serial_node;
	delete handshk_node;
	delete x_parser;

}
x_ret CXml::init(){			
	if(x_parser->Init() != R_XML_OK)
		return X_INIT_ERR;
		
	return X_RET_OK;
}

x_ret CXml::parseXml(){
	
	x_ret res;	
	
	//xml parsing using the xml schema input.xsd
	if(x_parser->Parse((char*)x_path, root_node) != R_XML_OK)
		return X_PARSE_ERR;
	
	//reading the xml style
	char ret[256];

	if(!x_parser->FindNodeRootat(root_node, (char*)"definition", def_node))
		return X_PARSE_ERR;
	else{
		//reading the title
		if (!x_parser->GetAttribute(def_node, (char*)"title", (char*)ret))
			return X_P_TITLE_ERR;
		else{
			*x_title = ret;
			#ifdef DEBUG
			printf("Title: %s\n", ret);
			#endif
		}
		
	
	}
	//Fill definition structure
	res = fillDefinition();

	if(res != X_RET_OK)
		return res;
	
	//Fill metadata structure
	return fillData();	

}

x_ret CXml::serial(){
	
	char ret[256];

	//reading the peripheral name (for example COM1)
	if (!x_parser->GetAttribute(stream_node, (char*)"stream_name", (char*)ret)){
		return X_P_STREAM_N_ERR;
	}else{
		#ifdef DEBUG
		printf("Stream name: %s\n", ret);
		#endif				
		x_def->s_Name = ret;
	}

	//reading the extra infos about the stream (channel number, port number, ...)
	if (x_parser->GetAttribute(stream_node, (char*)"stream_info", (char*)ret)){
		#ifdef DEBUG
		printf("Stream info: %d\n", x_def->s_Chan);
		#endif
		//conversion from char to int
		x_def->s_Chan = atoi(ret);
	}

	//serial presets
	if(!x_parser->FindNodeRootat(stream_node, (char*)"serial_stream", serial_node)){
		return X_P_S_STREAM_ERR;
	}else{
		//initializzation... creation of new struct Srs_232
		x_def->s_Preset = new struct Srs_232;

		//reading baud rate
		if (!x_parser->GetAttribute(serial_node, (char*)"baud_rate", (char*)ret)){
			return X_P_BAUD_RATE_ERR;
		}else{
			#ifdef DEBUG
			printf("Baud rate: %s\n", ret);
			#endif	
			x_def->s_Preset->rs_Baud = atoi(ret);			

		}

		//reading number of data bits
		if (!x_parser->GetAttribute(serial_node, (char*)"data_bits", (char*)ret)){
			return X_P_DATA_BITS_ERR;
		}else{
			#ifdef DEBUG
			printf("Data bits: %s\n", ret);
			#endif	
			x_def->s_Preset->rs_DBits = atoi(ret);			
		}

		//reading parity bit type 
		if (!x_parser->GetAttribute(serial_node, (char*)"parity", (char*)ret)){
			return X_P_PARITY_ERR;
		}else{
			#ifdef DEBUG
			printf("Parity bits: %s\n", ret);
			#endif	
			x_def->s_Preset->rs_Parity = ret;			
		}

		//reading stop bits
		if (!x_parser->GetAttribute(serial_node, (char*)"stop_bits", (char*)ret)){
			return X_P_STOP_ERR;
		}else{
			#ifdef DEBUG
			printf("Stop bits: %s\n", ret);
			#endif	
			x_def->s_Preset->rs_StopB = (float)atof(ret);			
		}
	}

	return X_RET_OK;
}

x_ret CXml::usb(){
	char ret[256];	

	//reading the peripheral name (for example USB1)
	if (!x_parser->GetAttribute(stream_node, (char*)"stream_name", (char*)ret)){
		return X_P_STREAM_N_ERR;
	}else{
		#ifdef DEBUG
		printf("Stream name: %s\n", ret);
		#endif				
		x_def->s_Name = ret;
	}
	
	//reading the extra infos about the stream (channel number, port number, ...)
	if (x_parser->GetAttribute(stream_node, (char*)"stream_info", (char*)ret)){
		#ifdef DEBUG
		printf("Stream info: %d\n", x_def->s_Chan);
		#endif
		//conversion from char to int
		x_def->s_Chan = atoi(ret);
	}
	return X_RET_OK;
}

x_ret CXml::file(){
	char ret[256];
	
	//reading the file name
	if (!x_parser->GetAttribute(stream_node, (char*) "stream_name", (char*)ret)){
		return X_P_STREAM_N_ERR;
	}else{
		x_def->s_Name = ret;
		#ifdef DEBUG
		printf("Stream name: %s\n", ret);
		#endif
	}
	//reading the file type 
	if (!x_parser->GetAttribute(stream_node, (char*)"stream_file_type", (char*)ret)){
		return X_P_STREAM_FT_ERR;
	}else{	
		#ifdef DEBUG
		printf("Stream file type: %s\n", ret);	
		#endif				
		if(!strcmp (ret, "text"))				//Generic text
			x_def->s_File_Type = T_TXT;
		else if(!strcmp (ret, "sniffer"))			//Sniffer
			x_def->s_File_Type = T_SNI;
		else
			return X_P_STREAM_FT_ERR;
	}
	return X_RET_OK;
}

x_ret CXml::pipe(){
	char ret[256];

	//reading the peripheral name (for example USB1)
	if (!x_parser->GetAttribute(stream_node, (char*)"stream_name", (char*)ret)){
		return X_P_STREAM_N_ERR;
	}else{
		#ifdef DEBUG
		printf("Stream name: %s\n", ret);
		#endif				
		x_def->s_Name = ret;
	}

	return X_RET_OK;
}

x_ret CXml::socket(){
	char ret[256];

	//reading the peripheral name (int this case IP-address)
	if (!x_parser->GetAttribute(stream_node, (char*)"stream_name", (char*)ret)){
		return X_P_STREAM_N_ERR;
	}else{
		#ifdef DEBUG
		printf("Stream name: %s\n", ret);
		#endif				
		x_def->s_Name = ret;
	}

	//reading the extra infos about the stream (channel number, port number, ...)
	if (!x_parser->GetAttribute(stream_node, (char*)"stream_info", (char*)ret)){
		return X_P_STREAM_C_ERR;
	}else{
		#ifdef DEBUG
		printf("Stream info: %d\n", x_def->s_Chan);
		#endif
		//conversion from char to int
		x_def->s_Sock_Port = atoi(ret);
	}

	return X_RET_OK;
}
/***************************************************************/
/*  ADD HERE THE FUNCTION FOR THE NEW COMMUNICATION DEVICE     */
/***************************************************************/

x_ret CXml::streamOption(){
	char ret[256];

	//stream option
	if(x_parser->FindNodeRootat(stream_node, (char*)"stream_opt", option_node)){
		//initializzation... creation of new struct SOption
		x_def->s_Option = new struct SOption;
		
		//reading option type
		if (!x_parser->GetAttribute(option_node, (char*)"s_type_opt", (char*)ret)){
			return X_P_O_TYPE_ERR;
		}else{
			#ifdef DEBUG
			printf("Stream option-character type: %s\n", ret);
			#endif	
			if(!strcmp (ret, "hex")){
				x_c_type = X_S_T_HEX;
			}else if(!strcmp (ret, "char")){
				x_c_type = X_S_T_CHAR;
			}
		}
		
		//reading the decoder type
		if (!x_parser->GetAttribute(option_node, (char*)"decoder", (char*)ret)){
			return X_P_O_START_ERR;
		}else{
			#ifdef DEBUG
			printf("Decoder type: %s\n", ret);
			#endif	
			if(!strcmp (ret, "none")){
				x_def->s_Option->o_Dec = D_NONE;
			}else if (strcmp(ret, "xor")){
				x_def->s_Option->o_Dec = D_XOR;
			}else if (strcmp(ret, "avr")){
				x_def->s_Option->o_Dec = D_AVR;
			}
			/***************************************************************/
			/*          ADD HERE THE CODE FOR THE NEW DECODER              */
			/***************************************************************/
			
		}

		//reading start char
		if (!x_parser->GetAttribute(option_node, (char*)"start_char", (char*)ret)){
			return X_P_O_START_ERR;
		}else{
			#ifdef DEBUG
			printf("Stream option-start character: %s\n", ret);
			#endif	
			switch(x_c_type){
				case X_S_T_HEX:
					x_def->s_Option->o_Start = dec2hex(ret[0], ret[1]);
					#ifdef DEBUG
					printf("Stream option-start character translated: %c\n", x_def->s_Option->o_Start);
					#endif	
				break;
				
				case X_S_T_CHAR:
					x_def->s_Option->o_Start = ret[0];
				break;
			}	
		}
		
		//reading end char
		if (!x_parser->GetAttribute(option_node, (char*)"end_char", (char*)ret)){
			return X_P_O_END_ERR;
		}else{
			#ifdef DEBUG
			printf("Stream option-end character: %s\n", ret);
			#endif	
			switch(x_c_type){
				case X_S_T_HEX:
					x_def->s_Option->o_End = dec2hex(ret[0], ret[1]);
					#ifdef DEBUG
					printf("Stream option-end character translated: %c\n", x_def->s_Option->o_End);
					#endif	
				break;
				
				case X_S_T_CHAR:
					x_def->s_Option->o_End = ret[0];
				break;
			return X_RET_OK;}				
		}
		if(x_def->s_Option->o_Dec != D_NONE){
			//reading escape char 
			if (x_parser->GetAttribute(option_node, (char*)"esc_char", (char*)ret)){
				#ifdef DEBUG
				printf("Stream option-escape character: %s\n", ret);
				#endif
				switch(x_c_type){
					case X_S_T_HEX:
						x_def->s_Option->o_Esc = dec2hex(ret[0], ret[1]);
						//#ifdef DEBUG
						printf("Stream option-escape character translated: %c\n", x_def->s_Option->o_Esc);
						//#endif	
					break;
					
					case X_S_T_CHAR:
						x_def->s_Option->o_Esc = ret[0];
					break;
				}							
			}else{
				if(x_def->s_Option->o_Dec == D_XOR){
					return X_P_O_ESC_ERR;
				}
			}
		}
	}
	return X_RET_OK;
}
x_ret CXml::lengthPacket(){
	char ret[256];

	//searching for packet description node
	if(!x_parser->FindNodeRootat(root_node, (char*)"packet_description", pkt_node)){
		return X_P_PKT_ERR;
	}else{
		//reading the packet size
		if (!x_parser->GetAttribute(pkt_node, (char*)"length_pkt", (char*)ret)){
			return X_P_PKT_S_ERR;
		}else if(!atoi(ret)){
			return X_P_PKT_S_ERR;
		}else{
			#ifdef DEBUG
			printf("Packet size: %s\n", ret);
			#endif
			x_def->s_Size = atoi(ret);
		}
	}
	
	return X_RET_OK;
}

x_ret CXml::handshake(){
	char ret[256];

	//searching for handshaking node
	if(!x_parser->FindNodeRootat(root_node, (char*)"handshake", handshk_node)){
		return X_P_HAND_ERR;
	}else{
		if(!x_parser->GetAttribute(handshk_node, (char*)"h_style", (char*)ret)){
			return X_P_H_STYLE_ERR;
		}else{
			#ifdef DEBUG
			printf("Handshake style: %s\n", ret);
			#endif
			if(!strcmp (ret, "none")){
				x_def->s_Style = H_S_NONE;
			}else if(!strcmp (ret, "standard")){
				if(x_def->s_Type == T_FILE)
					return X_GENERIC_ERR;
				else
					x_def->s_Style = H_S_STANDARD;
			}else if(!strcmp (ret, "start")){
				if(x_def->s_Type == T_FILE)
					return X_GENERIC_ERR;
				else
					x_def->s_Style = H_S_START;
			}
		}
		
		switch(x_def->s_Style){
			case H_S_NONE:
			x_def->s_Handshk = NULL;
			break;
				
			case H_S_START:
			//searching for handshaking node
			if(!x_parser->FindNodeRootat(handshk_node, (char*)"handshake_pkt", handshk_pkt_node)){
				return X_P_HAND_P_ERR;
			}else{
				//while for reading all sibling for and extract the handshking word
				x_def->s_Handshk = new vector<struct SHandshaking>;
				SHandshaking elem;
				vector<unsigned char> dummy;
					
				while(handshk_pkt_node != NULL){
					if (!x_parser->GetAttribute(handshk_pkt_node, (char*)"h_type_pkt", (char*)ret)){
						return X_P_H_SEND_ERR;
					}else{
						#ifdef DEBUG
						printf("Handshake option-character type: %s\n", ret);
						#endif	
						if(!strcmp (ret, "hex")){
							x_m_type = X_S_T_HEX;
						}else if(!strcmp (ret, "char")){
							x_m_type = X_S_T_CHAR;
						}
					}
					
					//read packet to send
					if (!x_parser->GetAttribute(handshk_pkt_node, (char*)"handshake_send", (char*)ret)){
						return X_P_H_SEND_ERR;
					}else{
						#ifdef DEBUG
						printf("Packet to send: %s\n", ret);
						#endif
						
						switch(x_m_type){
							case X_S_T_HEX: {
								int dim_v = strlen(ret);
								unsigned char dummy_t;

								if((dim_v % 2) != 0)
									return X_P_H_SEND_ERR;

								dummy.clear();
								
								for(int i = 0; i < dim_v; i+=2){
									dummy_t = dec2hex(ret[i], ret[i + 1]);
									dummy.push_back(dummy_t); 
								}
								
								elem.h_Size_P = dummy.size();
								elem.h_Pkt = new unsigned char[dummy.size()];
								
								for(unsigned int i = 0; i < dummy.size(); i++){
									#ifdef DEBUG
									printf("Packet to send translated: %c\n", dummy[i]);
									#endif	
									elem.h_Pkt[i] = dummy[i];
								}
								
								break;
							}
							case X_S_T_CHAR: {
								elem.h_Size_P = strlen(ret);
								elem.h_Pkt = new unsigned char[strlen(ret)];
								for(unsigned int i = 0; i < strlen(ret); i++){
									#ifdef DEBUG
									printf("Packet to send translated: %c\n",ret[i]);
									#endif	
									elem.h_Pkt[i] = ret[i];
								}			
								break;
							}
						}	
					}
					
					//reading ack (receive)
					if (x_parser->GetAttribute(handshk_pkt_node, (char*)"handshake_ack", (char*)ret)){
						#ifdef DEBUG	
						printf("Ack: %s\n", ret);
						#endif
						
						switch(x_m_type){
							case X_S_T_HEX: {
								int dim_v = strlen(ret);
								if((dim_v % 2) != 0)
									return X_P_H_ACK_ERR;
								
								dummy.clear();
								
								for(int i = 0; i < dim_v; i+=2){
									dummy.push_back(dec2hex(ret[i], ret[i + 1])); 
									
									if(dummy[i] == -1)
										return X_P_H_ACK_ERR;
								}
								elem.h_Size_A = dummy.size();
								elem.h_Ack = new unsigned char[dummy.size()];
								for(unsigned int i = 0; i < dummy.size(); i++){
									#ifdef DEBUG
									printf("Ack: %c\n", dummy[i]);
									#endif	
									elem.h_Ack[i] = dummy[i];
								}													
								break;
							}
							case X_S_T_CHAR: {
								elem.h_Size_A = strlen(ret);
								elem.h_Ack = new unsigned char[strlen(ret)];
								memcpy(elem.h_Ack, (unsigned char*)ret, strlen(ret) + 1);
								break;
							}
						}
					}else{
						elem.h_Size_A = 0;
						elem.h_Ack = NULL;
					}
						
				
					//reading nack (receive)
					if (x_parser->GetAttribute(handshk_pkt_node, (char*)"handshake_nack", (char*)ret)){
						#ifdef DEBUG	
						printf("Nack: %s\n", ret);
						#endif
						
						switch(x_m_type){
							case X_S_T_HEX: {
								int dim_v = strlen(ret);
								if((dim_v % 2) != 0)
									return X_P_H_NACK_ERR;
								
								dummy.clear();
								
								for(int i = 0; i < dim_v; i+=2){
									dummy.push_back(dec2hex(ret[i], ret[i + 1]));
									
									if(dummy[i] == -1)
										return X_P_H_NACK_ERR;
								}
								elem.h_Size_N = dummy.size();
								elem.h_Nack = new unsigned char[dummy.size()];
								
								for(unsigned int i = 0; i < dummy.size(); i++){
									#ifdef DEBUG
									printf("Nack translated: %c\n", dummy[i]);
									#endif	
									elem.h_Nack[i] = dummy[i];
								}
								elem.h_Nack[dummy.size()] = '\0';
								printf("\n");
								
								break;
							}
							
							case X_S_T_CHAR: {
								elem.h_Size_N = strlen(ret);
								elem.h_Nack = new unsigned char[strlen(ret)];
								memcpy(elem.h_Nack, (unsigned char*)ret, strlen(ret) + 1);
								break;
							}
						}
					}else{ 
						elem.h_Size_N = 0;
						elem.h_Nack = NULL;
					}
					
					// add element to vector x_meta
					x_def->s_Handshk->push_back(elem);
				
					//next child (data node)
					handshk_pkt_node = x_parser->Get_NextSibling(handshk_pkt_node);
				} //end while
			}
			break;
			
			case H_S_STANDARD: {
			
			x_def->s_Handshk = new vector<struct SHandshaking>;
			SHandshaking elem;
			string dummy;
			
			if(!x_parser->FindNodeRootat(handshk_node, (char*)"handshake_pkt", handshk_pkt_node)){
				return X_P_HAND_P_ERR;
			}else{
				if (!x_parser->GetAttribute(handshk_pkt_node, (char*)"h_type_pkt", (char*)ret)){
					return X_P_H_SEND_ERR;
				}else{
					#ifdef DEBUG
					printf("Handshake option-character type: %s\n", ret);
					#endif	
					if(!strcmp (ret, "hex")){
						x_m_type = X_S_T_HEX;
					}else if(!strcmp (ret, "char")){
						x_m_type = X_S_T_CHAR;
					}
				}
				
				//read packet to send
				if (!x_parser->GetAttribute(handshk_pkt_node, (char*)"handshake_send", (char*)ret)){
					return X_P_H_SEND_ERR;
				}else{
					#ifdef DEBUG
					printf("Packet to send: %s\n", ret);
					#endif
					
					switch(x_m_type){
						case X_S_T_HEX: {
							int dim_v = strlen(ret);
							unsigned char dummy_t;

							if((dim_v % 2) != 0)
								return X_P_H_SEND_ERR;

							dummy.clear();
							
							for(int i = 0; i < dim_v; i+=2){
								dummy_t = dec2hex(ret[i], ret[i + 1]);

								if(dummy_t == -1)
									return X_P_H_SEND_ERR;
								else
									dummy.push_back(dummy_t); 
							}
							
							elem.h_Size_P = dummy.size();
							
							elem.h_Pkt = new unsigned char[dummy.size()];
							
							for(unsigned int i = 0; i < dummy.size(); i++){
								#ifdef DEBUG
								printf("Packet to send translated: %c\n", dummy[i]);
								#endif	
								elem.h_Pkt[i] = dummy[i];
							}
							
						break;
						}
						
						case X_S_T_CHAR: {
							elem.h_Size_P = strlen(ret);
							elem.h_Pkt = new unsigned char[strlen(ret)];
							for(unsigned int i = 0; i < strlen(ret); i++){
								#ifdef DEBUG
								printf("Packet to send translated: %c\n",ret[i]);
								#endif	
								elem.h_Pkt[i] = ret[i];
							}			
						break;
						}
					}	
					
					elem.h_Ack = NULL;
					elem.h_Nack = NULL;
				}
			}
			
			x_def->s_Handshk->push_back(elem);
			
			break;
			}
		}
	}
	return X_RET_OK;
}
x_ret CXml::fillDefinition(){
	char ret[256];
	x_ret dummy_ret;
	//searching for stream node
	if(!x_parser->FindNodeRootat(root_node, (char*)"stream", stream_node)){
		return X_P_STREAM_ERR;
	}else{
		//reading the stream type
		if (!x_parser->GetAttribute(stream_node, (char*)"stream_type", (char*)ret)){
			return X_P_TITLE_ERR;
		}
		
		//initializzation... creation if they need
		x_def->s_Preset = NULL;
		x_def->s_Handshk = NULL;
		x_def->s_Option = NULL;
		
		#ifdef DEBUG
		printf("Stream type: %s\n", ret);
		#endif

		if(!strcmp (ret, "serial")){
			x_def->s_Type = T_SERIAL;
			
			dummy_ret = serial();
			if(dummy_ret != X_RET_OK)
				return dummy_ret;
			
		}else if(!strcmp (ret, "usb")){
			x_def->s_Type = T_USB;
			
			dummy_ret = usb();
			if(dummy_ret != X_RET_OK)
				return dummy_ret;
			
		}else if(!strcmp (ret, "file")){
			x_def->s_Type = T_FILE;
			
			dummy_ret = usb();
			if(dummy_ret != X_RET_OK)
				return dummy_ret;

		}else if(!strcmp (ret, "pipe")){
			x_def->s_Type = T_PIPE;

			dummy_ret = pipe();
			if(dummy_ret != X_RET_OK)
				return dummy_ret;

		}else if(!strcmp (ret, "socket")){
			x_def->s_Type = T_SOCKET;

			dummy_ret = socket();
			if(dummy_ret != X_RET_OK)
				return dummy_ret;

		/***************************************************************/
		/*    ADD HERE THE CODE FOR THE NEW COMMUNICATION DEVICE       */
		/***************************************************************/
	
		}else{
			return X_P_STREAM_T_ERR;  
		}
	}

	//stream option
	dummy_ret = streamOption();
	if(dummy_ret != X_RET_OK)
		return dummy_ret;
	
	//filling handshake structure
	dummy_ret = handshake();
	if(dummy_ret != X_RET_OK)
		return dummy_ret;

	//reading the packet size
	dummy_ret = lengthPacket();
	if(dummy_ret != X_RET_OK)
		return dummy_ret;
	
	return X_RET_OK;
}

var_type CXml::dataType(char* ret, unsigned long size){
	if(!strcmp (ret, "int8")){
		//... and validate data size
		if(size % INT8_S == 0){
			return T_INT8;
		}else{
			return T_NOTYPE;
		}
	}else if(!strcmp (ret, "uint8")){
		//... and validate data size
		if(size % INT8_S == 0){
			return T_UINT8;
		}else{
			return T_NOTYPE;
		}
	}else if(!strcmp (ret, "int16")){
		//... and validate data size
		if(size % INT16_S == 0){
			return T_INT16;
		}else{
			return T_NOTYPE;
		}
	}else if(!strcmp (ret, "uint16")){
		//... and validate data size
		if(size % INT16_S == 0){
			return T_UINT16;
		}else{
			return T_NOTYPE;
		}
	}else if(!strcmp (ret, "int32")){
		//... and validate data size
		if(size % INT32_S == 0){
			return T_INT32;
		}else{
			return T_NOTYPE;
		}
	}else if(!strcmp (ret, "uint32")){
		//... and validate data size
		if(size % INT32_S == 0){
			return T_UINT32;
		}else{
			return T_NOTYPE;
		}
	}else if(!strcmp (ret, "int64")){
		//... and validate data size
		if(size % INT64_S == 0){
			return T_INT64;
		}else{
			return T_NOTYPE;
		}
	}else if(!strcmp (ret, "uint64")){
		//... and validate data size
		if(size % INT64_S == 0){
			return T_UINT64;
		}else{
			return T_NOTYPE;
		}
	}else if(!strcmp (ret, "float32")){
		//... and validate data size
		if(size % FLOAT32_S == 0){
			return T_FLOAT32;
		}else{
			return T_NOTYPE;
		}
	}else if(!strcmp (ret, "float64")){
		//... and validate data size
		if(size % FLOAT64_S == 0){
			return T_FLOAT64;
		}else{
			return T_NOTYPE;
		}
	} else return T_NOTYPE;

	return T_NOTYPE;
}

data_semantic CXml::semanticType(char* ret){
	if(!strcmp (ret, "nothing")){
		return DS_NOTHING;
	}else if(!strcmp (ret, "image")){
		return DS_IMAGE;
	}else if(!strcmp (ret, "histogram")){
		return DS_HISTOGRAM;
	}else if(!strcmp (ret, "table")){
		return DS_TABLE;
	}

	return DS_NOTHING;
}

x_ret CXml::fillData(){

	SMData elem;
	char ret[256];
	unsigned long prev_pos = 0; 	//Previous data position
	unsigned long prev_size = 0; 	//Previous data size
	
	//check if the pkt_node is instantiated
	if(!pkt_node)
		return X_GENERIC_ERR;


	if(!x_parser->FindNodeRootat(pkt_node, (char*)"data", data_node)){
		return X_P_DTA_ERR;
	}else{
		//erase the vector, if it's full!
		if(!x_meta->empty()){
			x_meta->clear();
		}
		
		//read data
		while(data_node != NULL){
			//read data name
			if (!x_parser->GetAttribute(data_node, (char*)"data_name", (char*)ret)){
				return X_P_DTA_N_ERR;
			}else{
				#ifdef DEBUG
				printf("Data name returned: %s\n", ret);
				#endif				
				elem.md_Name = ret;
			}
			//read data position inside the packet
			if (!x_parser->GetAttribute(data_node, (char*)"data_position", (char*)ret)){
				return X_P_DTA_P_ERR;
			}else{
				#ifdef DEBUG	
				printf("Data position returned: %s\n", ret);
				#endif
				elem.md_Pos = atol(ret);
				if(prev_pos > elem.md_Pos)	//metadata vector not correctly sorted
					return X_VAL_MDATA_ERR;
			}
			
			//read data size
			if (!x_parser->GetAttribute(data_node, (char*)"data_size", (char*)ret)){
				return X_P_DTA_S_ERR;
			}else{
				#ifdef DEBUG
				printf("Data size returned: %s\n", ret);
				#endif				
				//set data size
				elem.md_Size = atol(ret);
				
				//read data type...
				if (!x_parser->GetAttribute(data_node, (char*)"data_type", (char*)ret)){
					return X_P_DTA_T_ERR;
				}else{
					#ifdef DEBUG
					printf("Data type returned: %s\n", ret);
					#endif
					
					elem.md_Type = dataType((char*)ret, elem.md_Size);
					if(elem.md_Type == T_NOTYPE)
						return X_VAL_MDATA_ERR;
												
				}
			}
			
			//position validation
			if(elem.md_Pos != prev_pos + prev_size){				
				return X_VAL_MDATA_ERR;
			}
			
			//read semantic type...
			if (!x_parser->GetAttribute(data_node, (char*)"type", (char*)ret)){
				return X_P_DTA_T_ERR;
			}else{
				#ifdef DEBUG
				printf("Semantic type returned: %s\n", ret);
				#endif
				elem.md_Sem = semanticType((char*)ret);						
			}
			
			// add element to vector x_meta
			x_meta->push_back(elem);
			
			//update previous position and size
			prev_pos = elem.md_Pos; 
			prev_size = elem.md_Size;
			
			//next child (data node)
			data_node = x_parser->Get_NextSibling(data_node);
		} //end while
	}
	
	return X_RET_OK;
}

unsigned char CXml::dec2hex(char hs, char ls){
	string dummy = "";
	
	dummy.push_back(hs);
	dummy.push_back(ls);
	istringstream stream1;
    stream1.str(dummy);
    int i;
	
    stream1 >> hex >> i;

	return (unsigned char) i;
  
}

